/*
 * Copyright (c) 2000 Ping Pan and Columbia University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by Columbia University
 * and its contributors.'' Neither the name of the University nor the names 
 * of its contributors may be used to endorse or promote products derived 
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef _IPOPT_VAR_H_
#define _IPOPT_VAR_H_

/*
 * IP Option Control Block:
 */
struct ipoptpcb {
	LIST_ENTRY(ipoptpcb) list;
	struct	socket *ocb_socket;	/* pointer to socket */
	struct	sockproto ocb_proto;	/* protocol family, protocol */
	int	ocb_flags;		/* socket options, etc. */
};

#define sotoipoptcb(so)		((struct ipoptpcb *)(so)->so_pcb)

/* space alloacted to a raw socket for IP option processing */
#define IPOPTSNDQ	8192
#define IPOPTRCVQ	8192

/* flags in ocb_flags: */
#define OCB_RECVIF	0x0001		/* record incoming if */
#define OCB_RECVRTCP	0x0002		/* recv RTCP */
#define OCB_RECVRSVP	0x0004		/* recv RSVP */

#define OCB_IF_SET(x)			((x)->ocb_flags &= OCB_RECVIF)
#define OCB_RTCP_SET(x)			((x)->ocb_flags &= OCB_RECVRTCP)
#define OCB_RSVP_SET(x)			((x)->ocb_flags &= OCB_RECVRSVP)

#define OCB_SET_RECVIF(x)		{(x)->ocb_flags |= OCB_RECVIF;}
#define OCB_SET_RTCP(x)			{(x)->ocb_flags |= OCB_RECVRTCP;}
#define OCB_SET_RSVP(x)			{(x)->ocb_flags |= OCB_RECVRSVP;}

#define OCB_BIT(x, bit)			((x)->ocb_flags & bit ? 1 : 0)


/* given we are not expecting a lot of IP options, we can live with
 * a chain list. In the future, we can change this to a hash table
 * if necessary.
 */

extern LIST_HEAD(ipoptcb_list_head, ipoptpcb) ipoptcb_list;


/* system-level info */
struct ipopt_stat {
	u_long	rr_count;
	u_long	ts_count;
	u_long	sec_count;
	u_long	lsrr_count;
	u_long	ssrr_count;
	u_long	ra_count;

	u_long	rsvp_count;
	u_long	rtcp_count;

	u_long	null_input;
	u_long	rr_input;
	u_long	ts_input;
	u_long	sec_input;
	u_long	lsrr_input;
	u_long	ssrr_input;
	u_long	ra_input;

	u_long	no_space;
	u_long	lost_input;
};

/*
 * Used for kernel-user system level communication.
 */
struct ipopt_mask {
	u_long	flag;		/* see below for bit mapping */
	u_long	alert_flag;
};

/* ipopt_mask: flag field */
#define IPOPT_RR_MASK		0x0001		/* 0000 0001 */
#define IPOPT_TS_MASK		0x0002		/* 0000 0010 */
#define IPOPT_SEC_MASK		0x0004		/* 0000 0100 */
#define IPOPT_LSRR_MASK		0x0008		/* 0000 1000 */
#define IPOPT_SSRR_MASK		0x0010		/* 0001 0000 */
#define IPOPT_RA_MASK		0x0020		/* 0010 0000 */

#define IPOPT_RR_OFFSET		0
#define IPOPT_TS_OFFSET		1
#define IPOPT_SEC_OFFSET	2
#define IPOPT_LSRR_OFFSET	3
#define IPOPT_SSRR_OFFSET	4
#define IPOPT_RA_OFFSET		5

#define IPOPT_MAX_OPTIONS	6

/* ipopt_mask: alert_flag field */
#define IPOPT_ARSVP_MASK	0x0001
#define IPOPT_ARTCP_MASK	0x0002


/* check the flag: zero or not */
#define IPOPT_RR_SET(x)		((x)->flag & IPOPT_RR_MASK)
#define IPOPT_TS_SET(x)		((x)->flag & IPOPT_TS_MASK)
#define IPOPT_SEC_SET(x)	((x)->flag & IPOPT_SEC_MASK)
#define IPOPT_LSRR_SET(x)	((x)->flag & IPOPT_LSRR_MASK)
#define IPOPT_SSRR_SET(x)	((x)->flag & IPOPT_SSRR_MASK)
#define IPOPT_RA_SET(x)		((x)->flag & IPOPT_RA_MASK)

#define IPOPT_ARSVP_SET(x)	((x)->alert_flag &= IPOPT_ARSVP_MASK)
#define IPOPT_ARTCP_SET(x)	((x)->alert_flag &= IPOPT_ARTCP_MASK)


/* set the flag */
#define IPOPT_SET_RR(x)		{(x)->flag |= IPOPT_RR_MASK;}
#define IPOPT_SET_TS(x)		{(x)->flag |= IPOPT_TS_MASK;}
#define IPOPT_SET_SEC(x)	{(x)->flag |= IPOPT_SEC_MASK;}
#define IPOPT_SET_LSRR(x)	{(x)->flag |= IPOPT_LSRR_MASK;}
#define IPOPT_SET_SSRR(x)	{(x)->flag |= IPOPT_SSRR_MASK;}
#define IPOPT_SET_RA(x)		{(x)->flag |= IPOPT_RA_MASK;}

#define IPOPT_SET_ARSVP(x)	{(x)->alert_flag |= IPOPT_ARSVP_MASK;}
#define IPOPT_SET_ARTCP(x)	{(x)->alert_flag |= IPOPT_ARTCP_MASK;}

/* clear the flag */
#define IPOPT_RESET_RR(x)	{(x)->flag &= ~IPOPT_RR_MASK;}
#define IPOPT_RESET_TS(x)	{(x)->flag &= ~IPOPT_TS_MASK;}
#define IPOPT_RESET_SEC(x)	{(x)->flag &= ~IPOPT_SEC_MASK;}
#define IPOPT_RESET_LSRR(x)	{(x)->flag &= ~IPOPT_LSRR_MASK;}
#define IPOPT_RESET_SSRR(x)	{(x)->flag &= ~IPOPT_SSRR_MASK;}
#define IPOPT_RESET_RA(x)	{(x)->flag &= ~IPOPT_RA_MASK;}

#define IPOPT_RESET_ARSVP(x)	{(x)->alert_flag &= ~IPOPT_ARSVP_MASK;}
#define IPOPT_RESET_ARTCP(x)	{(x)->alert_flag &= ~IPOPT_ARTCP_MASK;}


/* global parameters */
extern struct domain	ipoptdomain;
extern struct protosw	ipoptsw[];
extern struct pr_usrreqs	ipopt_usrreqs;
extern struct ipopt_stat	ipoptstat;
extern struct ipopt_mask	ipopt_cfg;
extern struct ipopt_mask	ipopt_pkt;
extern u_char ipopt_protox[IPOPT_MAX_OPTIONS];


/* func prototypes */
void	ipopt_init(void);
int	ipopt_ctloutput(struct socket *, struct sockopt *);
void	ipopt_map(int, u_char *, struct ip *);
int	ipopt_input(struct mbuf *);
void	null_input(struct mbuf *, int);
void	ra_input(struct mbuf *, int);
void	rr_input(struct mbuf *, int);
void	lsrr_input(struct mbuf *, int);
void	ssrr_input(struct mbuf *, int);
void	ts_input(struct mbuf *, int);
void	security_input(struct mbuf *, int);

void	ipopt_reg_proto(int, int);
void	ipopt_reset_cb(struct socket *);
int	ipopt_free_cb(struct socket *);
int	ipopt_accept_proto(int);
void	ipopt_proto_input(struct mbuf *, int, int);
void	ipopt_appenctl(struct ipoptpcb *, struct mbuf **, struct ip *, struct mbuf *);

#endif
